/*
 * Copyright (c) 2008-2020, Hazelcast, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hazelcast.test.starter.constructor;

import com.hazelcast.test.starter.HazelcastStarterConstructor;
import com.hazelcast.test.starter.answer.NodeEngineAnswer;

import java.lang.reflect.Constructor;

import static com.hazelcast.test.starter.ReflectionUtils.getFieldValueReflectively;
import static org.mockito.Mockito.mock;

@HazelcastStarterConstructor(classNames = {"com.hazelcast.spi.merge.SplitBrainMergePolicyProvider",
        "com.hazelcast.cache.impl.merge.policy.CacheMergePolicyProvider"})
public class MergePolicyProviderConstructor extends AbstractStarterObjectConstructor {

    public MergePolicyProviderConstructor(Class<?> targetClass) {
        super(targetClass);
    }

    @Override
    Object createNew0(Object delegate) throws Exception {
        ClassLoader classloader = targetClass.getClassLoader();
        Class<?> nodeEngineClass = classloader.loadClass("com.hazelcast.spi.NodeEngine");
        // obtain reference to constructor ...MergePolicyProvider(NodeEngine nodeEngine)
        Constructor<?> constructor = targetClass.getDeclaredConstructor(nodeEngineClass);

        Object nodeEngine = getFieldValueReflectively(delegate, "nodeEngine");
        Object[] args = new Object[]{mock(nodeEngineClass, new NodeEngineAnswer(nodeEngine))};

        return constructor.newInstance(args);
    }
}
